@@ -30,6 +30,7 @@ import contextlib
 import enum
 import hashlib
 import json
+import logging
 import time
 from typing import TYPE_CHECKING, Any, Generic, TypeVar, overload
 
@@ -40,6 +41,8 @@ from . import data as data_classes
 from . import errors, utils
 from .ratelimit import RateLimit
 
+logger = logging.getLogger(__name__)
+
 __all__ = (
     "QueryKit",
     "Query",
@@ -540,13 +543,18 @@ class QueryKit:
         )
 
     async def subscribe_internal(self, subscription: Subscription[Any]) -> None:
+        logger.debug("subscribe_internal - Subscribing to %s", subscription)
         if self.socket is None:
+            logger.debug("Creating new socket")
             self.socket = await Socket.connect(self)
         await self.socket.subscribe(subscription)
 
     async def unsubscribe_internal(self, subscription: Subscription[Any]) -> None:
+        logger.debug("Unsubscribing from %s", subscription)
         if self.socket is not None:
             await self.socket.unsubscribe(subscription)
+        else:
+            logger.debug("No socket to unsubscribe from")
 
     def get_response_errors(self, data: Dict[str, Any]) -> List[Dict[str, Any]]:
         if isinstance(data, list):
@@ -1308,8 +1316,10 @@ class Subscription(Generic[T]):
         callbacks : Callback[T]
             A list of async functions to call when an event is received
         """
+        logging.debug("subscribe -> Subscribing to %s %s %s", model, event, filters)
         self = cls(kit, model, event, filters)
         if callbacks:
+            logging.debug("Adding callbacks %s", callbacks)
             self.callbacks[:] = callbacks
         self.channel = await self.request_channel()
         await self.kit.subscribe_internal(self)
@@ -1326,6 +1336,7 @@ class Subscription(Generic[T]):
 
     async def request_channel(self) -> str:
         if self.kit.aiohttp_session is None:
+            logger.debug("Creating aiohttp session")
             self.kit.aiohttp_session = aiohttp.ClientSession()
         async with self.kit.aiohttp_session.request(
             "GET",
@@ -1335,6 +1346,9 @@ class Subscription(Generic[T]):
                 **self.filters_param,
             },
         ) as response:
+            logger.debug(
+                "Got response %s while requesting subscription channel", response
+            )
             data = await response.json()
             if data.get("error") is not None:
                 raise errors.SubscribeError(data["error"])
@@ -1425,7 +1439,9 @@ class Socket:
 
     @classmethod
     async def connect(cls, kit: QueryKit) -> Self:
+        logger.debug("Connecting to socket")
         if kit.aiohttp_session is None:
+            logger.debug("Creating aiohttp session")
             kit.aiohttp_session = aiohttp.ClientSession()
         ws = await kit.aiohttp_session.ws_connect(  # type: ignore
             kit.socket_url,
@@ -1435,21 +1451,27 @@ class Socket:
         )
         self = cls(kit, ws)
         self.run()
+        logger.debug("Socket connected")
         return self
 
     async def reconnect(self) -> None:
         if self.reconnecting:
+            logger.debug("Already reconnecting, ignoring")
             return
         else:
             self.reconnecting = True
+        logger.debug("Attempting to reconnect socket")
+
         # ensure the ping, pong loop doesn't close the connection due to thinking it timed out when the connection was broken and needs to be reconnected
         self.last_message = time.perf_counter()
         self.last_ping = time.perf_counter()
         self.last_pong = time.perf_counter() + 1
         try:
             if self.kit.aiohttp_session is None:
+                logger.debug("Creating aiohttp session")
                 self.kit.aiohttp_session = aiohttp.ClientSession()
             self.close_code = None
+            self.established.clear()
             self.ws = await self.kit.aiohttp_session.ws_connect(  # type: ignore
                 self.kit.socket_url,
                 max_msg_size=0,
@@ -1460,6 +1482,7 @@ class Socket:
                 subscription.succeeded.clear()
                 await self.subscribe(subscription)
         except BaseException as e:
+            logger.debug("Failed to reconnect socket", exc_info=e)
             self.reconnecting = False
             raise e
 
@@ -1471,7 +1494,7 @@ class Socket:
                         # message.type is Unknown
                         if message.type in {aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.CLOSING, aiohttp.WSMsgType.CLOSE}:  # type: ignore
                             await self.handle_socket_close()
-                        elif message.type not in {aiohttp.WSMsgType.TEXT, aiohttp.WSMsgType}:  # type: ignore
+                        elif message.type not in {aiohttp.WSMsgType.TEXT}:  # type: ignore
                             continue
                         # message.data is Unknown
                         ws_event = self.kit.loads(message.data)  # type: ignore
@@ -1506,20 +1529,30 @@ class Socket:
                         utils.print_exception_with_header(
                             "Encountered ConnectionResetError in socket", e
                         )
+                        logging.warning(
+                            "Encountered ConnectionResetError in socket", exc_info=e
+                        )
                         await self.close_and_reconnect()
                         break
                     except Exception as e:
                         utils.print_exception_with_header(
                             "Ignoring exception when parsing WebSocket message", e
                         )
+                        logging.warning(
+                            "Ignoring exception when parsing WebSocket message",
+                            exc_info=e,
+                        )
                 if self.ws.closed:
+                    logger.debug("actual_run -> Socket closed")
                     await self.handle_socket_close()
             except asyncio.TimeoutError as e:
                 utils.print_exception_with_header("Encountered exception in socket", e)
+                logging.warning("Encountered exception in socket", exc_info=e)
                 raise e
 
     async def handle_socket_close(self) -> None:
         close_code = self.ws.close_code or self.close_code
+        logger.debug("Socket closed with code %s", close_code)
         if close_code is None or close_code in range(4000, 4100):
             raise errors.NoReconnect(f"WebSocket closed with close code {close_code}")
         elif close_code in range(4100, 4200):
@@ -1529,6 +1562,7 @@ class Socket:
             await self.reconnect()
 
     async def close_and_reconnect(self, message: bytes = b"Pong timeout") -> None:
+        logger.debug("Closing and reconnecting")
         self.close_code = 1002
         with contextlib.suppress(ConnectionResetError):
             await self.ws.close(code=1002, message=message)
@@ -1547,6 +1581,7 @@ class Socket:
                 if self.last_ping > self.last_pong:
                     # if pinged, not received pong, passed timeout, close and reconnect
                     if self.last_ping + self.activity_timeout > time.perf_counter():
+                        logger.debug("ping_pong - Pong timeout")
                         # has pinged, has not received pong, and has not received any other messages in activity_timeout
                         # therefore, connection is dead and should be closed and reconnected
                         await self.close_and_reconnect()
@@ -1559,10 +1594,11 @@ class Socket:
                 # don't send ping if last ping was not ponged
                 if self.last_pong < self.last_ping:
                     continue
+                logger.debug("ping_pong - Sending ping")
                 await self.ws.send_json({"event": "pusher:ping", "data": {}})
                 self.last_ping = time.perf_counter()
-
             except ConnectionResetError:
+                logger.debug("ping_pong - Connection reset")
                 asyncio.create_task(self.close_and_reconnect(b"Connection reset"))
 
     def run(self) -> None:
@@ -1570,8 +1606,11 @@ class Socket:
         self.ping_pong_task = asyncio.create_task(self.ping_pong())
 
     async def authorize_subscription(self, subscription: Subscription[Any]) -> str:
+        if not self.established.is_set():
+            logger.debug("authorize_subscription - Waiting for connection to be spcket")
         await self.established.wait()
         if self.kit.aiohttp_session is None:
+            logger.debug("authorize_subscription - Creating aiohttp session")
             self.kit.aiohttp_session = aiohttp.ClientSession()
         async with self.kit.aiohttp_session.request(
             "POST",
@@ -1588,18 +1627,26 @@ class Socket:
         )
 
     async def send(self, event: str, data: Dict[str, Any]) -> None:
-        if self.ws.closed:
+        if self.closed:
+            logger.debug("Attempting to send but socket is closed")
             await self.close_and_reconnect()
         try:
             await self.ws.send_json({"event": event, "data": data})
         except ConnectionResetError:
+            logger.debug(
+                "Encountered ConnectionResetError while sending", exc_info=True
+            )
             await self.close_and_reconnect()
             await self.ws.send_json({"event": event, "data": data})
 
     async def subscribe(self, subscription: Subscription[Any]) -> None:
+        if self.closed:
+            logger.debug("Attempting to subscribe but socket is closed")
+            await self.close_and_reconnect()
         try:
             auth = await self.authorize_subscription(subscription)
         except errors.Unauthorized:
+            logger.debug("Channel is expired")
             subscription.channel = await subscription.request_channel()
             auth = await self.authorize_subscription(subscription)
         await self.send(
@@ -1614,10 +1661,16 @@ class Socket:
             self.subscriptions.discard(subscription)
             if subscription.channel is not None:
                 self.channels.pop(subscription.channel, None)
+            logger.debug("Subscription to %s did not succeed", subscription, exc_info=e)
             raise errors.SubscriptionDidNotSucceed() from e
+        logger.debug("Successfully subscribed to subscription %s", subscription)
 
     async def unsubscribe(self, subscription: Subscription[Any]) -> None:
         if subscription.channel is not None:
             self.channels.pop(subscription.channel, None)
             self.subscriptions.discard(subscription)
             await self.send("pusher:unsubscribe", {"channel": subscription.channel})
+
+    @property
+    def closed(self) -> bool:
+        return self.ws.closed or self.close_code is not None
